/*
 * webserver.cpp
 *
 *  Created on: Mar 5, 2010
 *      Author: ssmethurst
 */

#include "webserver.h"

CWebserver Webserver;
Server port_eighty(80);

CWebserver::CWebserver(){
}
void CWebserver::Start() {
	port_eighty.begin();
}

bool CWebserver::ProcessHeader( char * requestBuffer, CWebserverRequest * request )
{
	if( request == NULL || request == NULL ) {
		return false;
	}

	sprintf( request->request_string, "/" );

	// Get the request path.
	char * get = strstr( requestBuffer, "GET " );
	if( get != NULL ) {
		get += 4 ;
		char * eol = strstr( get, " " );
		if( eol != NULL ) {
			eol[ 0 ] = 0 ;
		}
		unsigned short length = strlen( get );
		if( length > HTTP_MAX_REQUEST_LENGTH ) {
			length = HTTP_MAX_REQUEST_LENGTH-1;
		}
		strncpy( request->request_string, get, length );
		request->request_string[ length ] = 0 ; // NULL termanate the string.
	}

	return true;
}

void CWebserver::Respond( Client * client, char * requestBuffer )
{
	if( client == NULL || requestBuffer == NULL ) {
		return ;
	}

	CWebserverRequest request;
	if( ! this->ProcessHeader( requestBuffer, &request ) ) {
		return ; // Can not process header.
	}
	if( strcmp( request.request_string, "/xml/" ) == 0 ) {
		// Send a standard http response header
		client->print( "HTTP/" );
		client->print( HTTP_VERSION );
		client->print( " " );
		client->println( HTTP_STATUS_CODE_OK );

		client->println("Content-type: text/xml");
		client->println();

		client->println("<?xml version=\"1.0\"?>" );
		client->print("<relay>" );
		client->print( digitalRead(RELAY_PIN) );
		client->println("</relay>" );


	} else if( strcmp( request.request_string, "/" ) == 0 ) {
		// Send a standard http response header
		client->print( "HTTP/" );
		client->print( HTTP_VERSION );
		client->print( " " );
		client->println( HTTP_STATUS_CODE_OK );

		// HTTP header
		client->println("Content-Type: text/html");
		client->println();

		// content
		client->print("Relay status: ");
		if( digitalRead(RELAY_PIN) == 1 ) {
			client->print( "On" );
		} else {
			client->print( "Off" );
		}

		// Footer
		client->println( "\n<br /><br /><br />\n" );
		client->print( "Request String: ");
		client->println( request.request_string );
		client->println( "<br />");
		client->println("Page generated by <a href='http://www.chipkin.com/'>Chipkin Automation Systems</a>, BACnet Arduino v0.01");
	} else {
		// Send a standard http response header
		client->print( "HTTP/" );
		client->print( HTTP_VERSION );
		client->print( " " );
		client->println( HTTP_STATUS_CODE_NOT_FOUND );

	}
}

void CWebserver::ProcessRequest( Client * client )
{
	if( client == NULL ) {
		return ;
	}

	char buf[ HTTP_MAX_BUFFER_SIZE ];
	unsigned short length = 0 ;

	// an http request ends with a blank line
	boolean current_line_is_blank = true;
	while (client->connected())
	{
		if (client->available())
		{
			char c = client->read();
			Serial.print(c) ;
			if( length < HTTP_MAX_BUFFER_SIZE ) {
				buf[ length++ ] = c ;
			}

			// if we've gotten to the end of the line (received a newline
			// character) and the line is blank, the http request has ended,
			// so we can send a reply
			if (c == '\n' && current_line_is_blank)
			{
				this->Respond( client, buf );
				delay(1);
				break;
			}
			if (c == '\n') {
				// we're starting a new line
				current_line_is_blank = true;
			} else if (c != '\r') {
				// we've gotten a character on the current line
				current_line_is_blank = false;
			}
		}
	}
}

void CWebserver::Loop() {

	Client client = port_eighty.available();
	if (client)
	{
		this->ProcessRequest( &client );
		// give the web browser time to receive the data
		client.stop();
	}
}
